/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.fab.timestamp;

import org.bioimageanalysis.icy.common.geom.point.Point5D;
import org.bioimageanalysis.icy.common.math.MathUtil;
import org.bioimageanalysis.icy.common.math.UnitUtil;
import org.bioimageanalysis.icy.common.string.StringUtil;
import org.bioimageanalysis.icy.gui.GraphicsUtil;
import org.bioimageanalysis.icy.gui.canvas.IcyCanvas;
import org.bioimageanalysis.icy.model.overlay.Overlay;
import org.bioimageanalysis.icy.model.sequence.Sequence;
import org.bioimageanalysis.icy.system.logging.IcyLogger;
import org.jetbrains.annotations.NotNull;

import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.geom.Rectangle2D;
import java.util.concurrent.TimeUnit;

public class TimeStampOverlay extends Overlay {
    enum DisplayStyle {
        FRAME, TIME
    }

    String timeStampText;
    int x;
    int y;
    Font font;
    DisplayStyle displayStyle = DisplayStyle.FRAME;
    Rectangle2D textBound = new Rectangle2D.Double(0, 0, 0, 0);
    int fontSize = 20;
    int displaySelectionIndex = 0;

    public TimeStampOverlay(final @NotNull Sequence sequence) {
        super("Timestamp");

        x = 20;
        y = sequence.getHeight() - 20;
        font = new Font("Arial", Font.BOLD, fontSize);
    }

    /**
     * Display the time with all the units.
     *
     * @param valueInMs : value in milliseconds
     * @param displayMs : Even if a unit is not relevant (equals to zero), it will be displayed.
     * @return <b>Example:</b> "2h 3min 40sec 350ms".
     */
    public static String getTimeAsString(final double valueInMs, final boolean displayMs) {
        double v = Math.abs(valueInMs);

        final int day = (int) (v / (24d * 60d * 60d * 1000d));
        v %= 24d * 60d * 60d * 1000d;
        final int hour = (int) (v / (60d * 60d * 1000d));
        v %= 60d * 60d * 1000d;
        final int minute = (int) (v / (60d * 1000d));
        v %= 60d * 1000d;
        final int second = (int) (v / 1000d);
        v %= 1000d;
        final double ms = MathUtil.round(v, 1);

        String result = "";

        if (day > 0)
            result += day + "d ";
        if (hour > 0)
            result += String.format("%2dh ", hour);
        else if (!StringUtil.isEmpty(result))
            result += "00h ";
        if (minute > 0)
            result += String.format("%2dmin ", minute);
        else if (!StringUtil.isEmpty(result))
            result += "00min ";
        if (second > 0)
            result += String.format("%2dsec ", second);
        else if (!StringUtil.isEmpty(result))
            result += "00sec ";
        if (ms > 0d)
            result += String.format("%05.1fms", ms);
        else if (displayMs && !StringUtil.isEmpty(result))
            result += String.format("%05.1fms", 0d);

        return result;
    }

    @Override
    public void paint(final Graphics2D g, final Sequence sequence, final IcyCanvas canvas) {
        if (g == null)
            return;

        g.setColor(Color.white);
        g.setFont(font);

        switch (displayStyle) {
            case FRAME:
                timeStampText = "Current frame: " + canvas.getPositionT();
                break;

            case TIME:
                final double timeIntervalMs = sequence.getTimeInterval() * 1000d;
                final TimeUnit unit = UnitUtil.getBestTimeUnit(timeIntervalMs * 10);
                final double ms = canvas.getPositionT() * timeIntervalMs;

                // minute or "higher" time unit ?
                if (unit.ordinal() >= TimeUnit.MINUTES.ordinal())
                    timeStampText = getTimeAsString(ms, (timeIntervalMs % 1000d) != 0d);
                else
                    timeStampText = UnitUtil.displayTimeAsStringWithComma(ms, 3, unit);
                break;
        }

        g.drawString(timeStampText, x, y);

        final Rectangle2D r = GraphicsUtil.getStringBounds(g, font, timeStampText);
        r.setRect(r.getX() + x, r.getY() + y, r.getWidth(), r.getHeight());
        textBound = r;
    }

    @Override
    public void keyPressed(final KeyEvent e, final Point5D.Double imagePoint, final IcyCanvas canvas) {
        if (e.isConsumed())
            return;

        if (e.getKeyChar() == 'm') {
            x = (int) imagePoint.getX();
            y = (int) imagePoint.getY();
            canvas.getSequence().overlayChanged(this);
            return;
        }

        // if ( !isPointInTextArea( imagePoint ) ) return;

        if (e.getKeyChar() == 't') {
            displaySelectionIndex++;
            if (displaySelectionIndex >= DisplayStyle.values().length) {
                displaySelectionIndex = 0;
            }

            displayStyle = DisplayStyle.values()[displaySelectionIndex];

        }

        if (e.getKeyChar() == '+') {
            fontSize++;
        }

        if (e.getKeyChar() == '-') {
            fontSize--;
            if (fontSize < 2)
                fontSize = 2;
        }

        if (e.getKeyChar() == 'k') {
            canvas.getSequence().removeOverlay(this);
            IcyLogger.info(this.getClass(), "Time Stamp removed");
        }

        font = new Font("Arial", Font.BOLD, fontSize);
        canvas.getSequence().overlayChanged(this);

        e.consume();
    }
}
